package eu.dnetlib.services.oai.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import eu.dnetlib.common.controller.DnetRestController;
import eu.dnetlib.domain.oai.OaiConfiguration;
import eu.dnetlib.domain.oai.OaiMetadataFormat;
import eu.dnetlib.domain.oai.OaiSet;
import eu.dnetlib.errors.DnetException;
import eu.dnetlib.services.oai.service.OaiService;
import io.swagger.v3.oas.annotations.parameters.RequestBody;

@RestController
@RequestMapping("/api/oai")
public class ApiController extends DnetRestController {

	@Autowired
	private OaiService oaiService;

	@GetMapping("/conf")
	public OaiConfiguration oaiConfiguration() throws DnetException {
		return oaiService.oaiConfiguration();
	}

	@PostMapping("/conf")
	public OaiConfiguration oaiConfiguration(@RequestBody final OaiConfiguration conf) throws DnetException {
		return oaiService.oaiConfiguration(conf);
	}

	@GetMapping("/md-formats")
	public List<OaiMetadataFormat> listMdFormats() throws DnetException {
		return oaiService.listMetadataFormats();
	}

	@PostMapping("/md-formats")
	public List<OaiMetadataFormat> addOrUpdateMdFormats(@RequestBody final OaiMetadataFormat mdFormat) throws DnetException {
		oaiService.addOrUpdateMetadataFormat(mdFormat);
		return oaiService.listMetadataFormats();
	}

	@DeleteMapping("/md-formats/{mdprefix}")
	public List<OaiMetadataFormat> deleteMdFormats(@PathVariable final String mdprefix) throws DnetException {
		oaiService.deleteMetadataFormat(mdprefix);
		return oaiService.listMetadataFormats();
	}

	@GetMapping("/sets")
	public List<OaiSet> listOaiSets() throws DnetException {
		return oaiService.listSets();
	}

	@PostMapping("/sets")
	public List<OaiSet> addOrUpdateOaiSet(@RequestBody final OaiSet oaiSet) throws DnetException {
		oaiService.addOrUpdateSet(oaiSet);
		return oaiService.listSets();
	}

	@DeleteMapping("/sets/{setSpec}")
	public List<OaiSet> deleteOaiSet(@PathVariable final String setSpec) throws DnetException {
		oaiService.deleteSet(setSpec);
		return oaiService.listSets();
	}
}
