package eu.dnetlib.wfs;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.domain.EntityScan;
import org.springframework.context.annotation.Bean;
import org.springframework.scheduling.annotation.EnableScheduling;

import eu.dnetlib.common.app.AbstractDnetApp;
import eu.dnetlib.common.clients.DnetServiceClientFactory;
import eu.dnetlib.common.index.solr.SolrService;
import eu.dnetlib.common.mapping.cleaner.CleanerFactory;
import eu.dnetlib.common.mapping.xslt.DnetXsltFunction;
import eu.dnetlib.common.mapping.xslt.XsltTransformFactory;
import eu.dnetlib.common.mdstores.backends.sql.MDStoreSqlBackend;
import eu.dnetlib.domain.service.ServiceType;
import eu.dnetlib.domain.wfs.conf.WfConfiguration;
import eu.dnetlib.domain.wfs.conf.WfSection;
import eu.dnetlib.domain.wfs.jobs.WfJournalEntry;
import eu.dnetlib.domain.wfs.subscriptions.WfSubscription;

@SpringBootApplication
@EntityScan(basePackageClasses = {
		WfJournalEntry.class,
		WfConfiguration.class,
		WfSection.class,
		WfSubscription.class
})
@EnableScheduling
public class WfExecutorApplication extends AbstractDnetApp {

	@Autowired(required = false)
	private DnetXsltFunction[] xsltFunctions;

	@Value("${mdstores.data.datasource.url}")
	private String databaseUrl;

	@Value("${mdstores.data.datasource.username}")
	private String databaseUsername;

	@Value("${mdstores.data.datasource.password}")
	private String databasePassword;

	@Value("${solr.urls}")
	private String[] solrUrls;

	@Value("${zookeeper.server}")
	private String zooServer;

	public static void main(final String[] args) {
		SpringApplication.run(WfExecutorApplication.class, args);
	}

	@Override
	protected ServiceType serviceType() {
		return ServiceType.wf_executor;
	}

	@Bean
	public MDStoreSqlBackend mdStoreSqlBackend() {
		return new MDStoreSqlBackend(databaseUrl, databaseUsername, databasePassword);
	}

	@Bean
	public XsltTransformFactory xsltTransformFactory(final DnetServiceClientFactory clientFactory) {
		return new XsltTransformFactory(clientFactory, xsltFunctions);
	}

	@Bean
	public CleanerFactory cleanerFactory(final DnetServiceClientFactory clientFactory) {
		return new CleanerFactory(clientFactory);
	}

	@Bean
	public SolrService solrService() {
		return new SolrService(zooServer, solrUrls);
	}

}
