package eu.dnetlib.common.mapping.xslt;

import java.io.StringWriter;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;

import javax.xml.transform.stream.StreamSource;

import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import eu.dnetlib.common.mapping.RecordTransformer;
import net.sf.saxon.s9api.ExtensionFunction;
import net.sf.saxon.s9api.Processor;
import net.sf.saxon.s9api.QName;
import net.sf.saxon.s9api.SaxonApiException;
import net.sf.saxon.s9api.Serializer;
import net.sf.saxon.s9api.XdmAtomicValue;
import net.sf.saxon.s9api.XdmNode;
import net.sf.saxon.s9api.XsltCompiler;
import net.sf.saxon.s9api.XsltExecutable;
import net.sf.saxon.s9api.XsltTransformer;

public class XsltUtils {

	private static final Log log = LogFactory.getLog(XsltUtils.class);

	public static String applyXslt(final String xml, final String xslt) throws Exception {
		return applyXslt(xml, xslt, new HashMap<>());
	}

	public static String applyXslt(final String xml, final String xslt, final Map<String, Object> params, final DnetXsltFunction... xsltFunctions)
			throws Exception {
		final RecordTransformer<String, String> transformer = newTransformer(xslt, params, xsltFunctions);
		return transformer.transform(xml);
	}

	public static RecordTransformer<String, String> newTransformer(final String xsltText,
			final Map<String, Object> params,
			final DnetXsltFunction... xsltFunctions) throws Exception {

		final Processor processor = new Processor(false);
		for (final DnetXsltFunction f : xsltFunctions) {
			final ExtensionFunction extFunction = f.asExtensionFunction();
			processor.registerExtensionFunction(extFunction);
			log.info("New XSLT function registered: " + extFunction.getName());
		}

		final XsltCompiler comp = processor.newXsltCompiler();
		params.forEach((k, v) -> comp.setParameter(new QName(k), XdmAtomicValue.makeAtomicValue(v)));

		final XsltExecutable xslt = comp
				.compile(new StreamSource(IOUtils.toInputStream(xsltText, StandardCharsets.UTF_8)));

		return xml -> {
			try {
				final XdmNode source = processor
						.newDocumentBuilder()
						.build(new StreamSource(IOUtils.toInputStream(xml, StandardCharsets.UTF_8)));

				final XsltTransformer trans = xslt.load();
				trans.setInitialContextNode(source);

				final StringWriter output = new StringWriter();
				final Serializer out = processor.newSerializer(output);
				out.setOutputProperty(Serializer.Property.METHOD, "xml");
				out.setOutputProperty(Serializer.Property.INDENT, "yes");

				trans.setDestination(out);
				trans.transform();

				return output.toString();
			} catch (final SaxonApiException e) {
				throw new RuntimeException(e);
			}
		};
	}

}
